#import <Foundation/Foundation.h>
#import "Common.h"
#import "Config.h"

static int reader = 0;
static int writer = 0;
static int msgIdx = 0;
static int tCount = NUM_READERS + NUM_WRITERS;

static NSMutableString *gMessage;
static NSLock *gLock;

static NSString *msgs[] = {
	@"Hello World!", @"how are you?", @"random message!", @"hope we have enough of these...",
	@"I'm sure we will", @"there so many!", @"How many messages does it take", @"to screw in a lightbulb?"
};

#define newMsgIdx (msgIdx++%(sizeof(msgs)/sizeof(msgs[0])))

@interface Actor : NSObject
- (void)readerMain;
- (void)writerMain;
@end

@implementation Actor
- (void)readerMain
{
	NSAutoreleasePool *pool = [NSAutoreleasePool new];
	
	int i=READ_TIMES, readerID = ++reader;
	
	while ( --i > 0 )
	{
		fprintf(stderr, "Reader %d getting message...\n", readerID);
		[gLock lock];
		fprintf(stderr, "Reader %d got message: %s\n", readerID, [gMessage UTF8String]);
		usleep(READ_SLEEP);
		[gLock unlock];
	}
	
	fprintf(stderr, "Reader %d done!\n", readerID);
	if ( --tCount == 0 )
	{
		printf("good-bye\n");
		exit(0);
	}
	[pool release];
}
- (void)writerMain
{
	NSAutoreleasePool *pool = [NSAutoreleasePool new];
	
	int i=WRITE_TIMES, writerID = ++writer;
	
	while ( --i > 0 )
	{
		fprintf(stderr, "Writer %d getting message...\n", writerID);
		[gLock lock];
		[gMessage setString:msgs[newMsgIdx]];
		fprintf(stderr, "Writer %d set message to: %s\n", writerID, [gMessage UTF8String]);
		usleep(WRITE_SLEEP);		
		[gLock unlock];
	}
	
	fprintf(stderr, "Writer %d done!\n", writerID);
	if ( --tCount == 0 )
	{
		printf("good-bye\n");
		exit(0);
	}
	[pool release];
}
@end


int main(int argc, char const *argv[])
{
	NSAutoreleasePool *pool = [NSAutoreleasePool new];
	
	gMessage = [[NSMutableString alloc] initWithString:msgs[newMsgIdx]];
	gLock = [NSLock new];
	
	for (int i=0; i<NUM_READERS; ++i)
		[NSThread detachNewThreadSelector:@selector(readerMain) toTarget:[Actor new] withObject:nil];
	for (int i=0; i<NUM_WRITERS; ++i)
		[NSThread detachNewThreadSelector:@selector(writerMain) toTarget:[Actor new] withObject:nil];
	
	// wait for threads to terminate...
	while (tCount)
		sleep(1);
		
	printf("good-bye\n");
	[pool release];
	return 0;
}